@@ -1,25 +1,21 @@
-#[derive(Copy, Clone)]
+/// The type for a [RateLimit](super::RateLimit). Either a rate limit for the
+/// entire app (`Application`) or for a specific method (`Method`).
+/// Method rate limit will handle service violations.
+#[derive(Debug, Copy, Clone, PartialEq, Eq)]
 pub enum RateLimitType {
     Application,
     Method,
 }
 
 impl RateLimitType {
-    pub fn type_name(self) -> &'static str {
-        match self {
-            Self::Application => "application",
-            Self::Method => "method",
-        }
-    }
-
-    pub fn limit_header(self) -> &'static str {
+    pub const fn limit_header(self) -> &'static str {
         match self {
             Self::Application => "X-App-Rate-Limit",
             Self::Method => "X-Method-Rate-Limit",
         }
     }
 
-    pub fn count_header(self) -> &'static str {
+    pub const fn count_header(self) -> &'static str {
         match self {
             Self::Application => "X-App-Rate-Limit-Count",
             Self::Method => "X-Method-Rate-Limit-Count",
